# Python Pan123
# 在使用前，请去123云盘开放平台(https://www.123pan.cn/developer)申请使用权限
# 在邮箱中查询client_id和client_secret，并使用get_access_token函数获取访问令牌

from .abstracts import Requestable

from .share import Share
from .file import File
from .user import User
from .offline_download import OfflineDownload
from .direct_link import DirectLink
from .transcode import Transcode
from .oss import OSS
from .utils.dict_util import merge_dict


class Pan123(Requestable):
    def __init__(
        self,
        access_token: str,
        base_url: str = "https://open-api.123pan.com",
        header: dict = {
            "Content-Type": "application/json",
            "Platform": "open_platform",
        },
    ):
        super().__init__(
            base_url,
            merge_dict(header, {"Authorization": f"Bearer {access_token}"}),
        )
        self.share = Share(self.base_url, self.header)
        self.file = File(self.base_url, self.header)
        self.user = User(self.base_url, self.header)
        self.offline_download = OfflineDownload(self.base_url, self.header)
        self.direct_link = DirectLink(self.base_url, self.header)
        self.transcode = Transcode(self.base_url, self.header)
        self.oss = OSS(self.base_url, self.header)



def merge_dict(dict1: dict, dict2: dict) -> dict:
    result = dict1.copy()
    for key, value in dict2.items():
        if key in result and isinstance(result[key], dict) and isinstance(value, dict):
            result[key] = merge_dict(result[key], value)
        else:
            result[key] = value
    return result



import requests

from .utils.request import parse_response_data
from .abstracts import Requestable
from .costants import SearchMode, VideoFileType


class Transcode(Requestable):
    def folder_info(self, file_id: int) -> dict:
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/folder/info"),
                headers=self.header,
                data={"fileID": file_id},
            )
        )

    def file_list(
        self,
        parent_file_id: int,
        limit: int,
        search_data: str = "",
        search_mode: SearchMode = SearchMode.NORMAL,
        last_file_id: int = 0,
    ) -> list[dict]:
        data: dict = {
            "parentFileId": parent_file_id,
            "limit": limit,
            "businessType": 2,
        }
        if search_data:
            data["searchData"] = search_data
        if search_mode:
            data["searchMode"] = search_mode
        if last_file_id:
            data["lastFileId"] = last_file_id
        return parse_response_data(
            requests.post(
                self.use_url("/api/v2/file/list"),
                data=data,
                headers=self.header,
            )
        )

    def from_cloud_disk(self, file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/upload/from_cloud_disk"),
                data={"fileId": file_id},
                headers=self.header,
            )
        )

    def delete(self, file_id: int, original: bool = False, transcoded: bool = False):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/delete"),
                data={
                    "fileId": file_id,
                    "businessType": 2,
                    "trashed": original + transcoded,
                },
                headers=self.header,
            )
        )

    def video_resolution(self, file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/video/resolution"),
                data={"fileId": file_id},
                headers=self.header,
            )
        )

    def video(
        self,
        file_id: int,
        codec_name: str,
        video_time: str,
        resolutions: list[int],
    ):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/video"),
                data={
                    "fileId": file_id,
                    "codecName": codec_name,
                    "videoTime": video_time,
                    "resolutions": ",".join(map(lambda x: f"{x}P", resolutions)),
                },
                headers=self.header,
            )
        )

    def video_record(self, file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/video/record"),
                data={"fileId": file_id},
                headers=self.header,
            )
        )

    def video_result(self, file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/video/result"),
                data={"fileId": file_id},
                headers=self.header,
            )
        )

    def file_download(self, file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/file/download"),
                data={"fileId": file_id},
                headers=self.header,
            )
        )

    def m3u8_ts_download(
        self,
        file_id: int,
        resolution: int,
        file_type: VideoFileType,
        ts_name: str = "",
    ):
        data = {
            "fileId": file_id,
            "resolution": f"{resolution}P",
            "type": file_type,
        }
        if ts_name:
            data["tsName"] = ts_name
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/m3u8_ts/download"),
                data=data,
                headers=self.header,
            )
        )

    def file_download_all(self, file_id: int, zip_name: str):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/transcode/file/download_all"),
                data={
                    "fileId": file_id,
                    "zipName": zip_name,
                },
                headers=self.header,
            )
        )



import json

from typing import Any, Type
from requests import Response, HTTPError
from .exceptions import CloudError


def parse_response_data(
    response: Response,
    fail_raise: Type[Exception] = CloudError,
) -> Any:
    if response.status_code == 200:
        response_data = json.loads(response.text)
        if response_data["code"] == 0:
            return response_data["data"]
        else:
            raise fail_raise(response_data)
    else:
        raise HTTPError(response.text)



import requests

from .utils.request import parse_response_data
from .abstracts import Requestable


class OSSSourceCopy(Requestable):
    def copy(self, file_ids: list[int], to_parent_file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/oss/source/copy"),
                data={
                    "fileIDs": file_ids,
                    "toParentFileID": to_parent_file_id,
                    "sourceType": 1,
                    "type": 1,
                },
                headers=self.header,
            )
        )

    def fail(self, task_id: str, limit: int = 1, page: int = 0):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/oss/source/copy/fail"),
                data={
                    "taskID": task_id,
                    "limit": limit,
                    "page": page,
                },
                headers=self.header,
            )
        )

    def process(self, task_id: str):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/oss/source/copy/process"),
                data={"taskID": task_id},
                headers=self.header,
            )
        )



import requests

from .utils.request import parse_response_data
from .abstracts import Requestable


class DirectLink(Requestable):
    def doPost(self, url: str, data: dict):
        return parse_response_data(
            requests.post(
                self.use_url(f"/api/v1/direct-link/{url}"),
                data=data,
                headers=self.header,
            )
        )

    def query_transcode(self, ids: list):
        return self.doPost("queryTranscode", {"ids": ids})

    def do_transcode(self, ids):
        return self.doPost("doTranscode", {"ids": ids})

    def get_m3u8(self, file_id):
        return self.doPost("get/m3u8", {"fileID": file_id})

    def enable(self, file_id):
        return self.doPost("enable", {"fileID": file_id})

    def disable(self, file_id):
        return self.doPost("disable", {"fileID": file_id})

    def list_url(self, file_id):
        return self.doPost("url", {"fileID": file_id})



class ClientKeyError(Exception):
    def __init__(self, r):
        self.r = r
        super().__init__(f"错误的client_id或client_secret，请检查后重试\n{self.r}")


class AccessTokenError(Exception):
    def __init__(self, r):
        self.r = r
        super().__init__(f"错误的access_token，请检查后重试\n{self.r}")


class CloudError(Exception):
    def __init__(self, r):
        self.r = r
        super().__init__(f"{self.r['message']}")


class PacketLossError(Exception):
    def __init__(self, index: int):
        self.index = index
        super().__init__(f"第{index+1}个分片上传失败，可能是网络问题，请重试")



import hashlib, requests, os, math

from .utils.exceptions import PacketLossError
from .utils.request import parse_response_data
from .utils.file_metadata import get_file_md5
from .abstracts import Requestable
from .costants import SearchMode, DuplicateMode
from typing import Literal


class File(Requestable):
    def legacy_list_file(
        self,
        parent_file_id: int,
        page: int = 0,
        limit: int = 0,
        order_by: str = "file_id",
        order_direction: Literal["asc", "desc"] = "asc",
        trashed: bool = False,
        search_data: str = "",
    ) -> list[dict]:
        data: dict = {"parentFileId": parent_file_id}
        if page:
            data["page"] = page
        if limit:
            data["limit"] = limit
        if order_by:
            data["orderBy"] = order_by
        if order_direction:
            data["orderDirection"] = order_direction
        if trashed:
            data["trashed"] = trashed
        if search_data:
            data["searchData"] = search_data
        return parse_response_data(
            requests.get(
                self.use_url("/api/v1/file/list"),
                data=data,
                headers=self.header,
            )
        )

    def list_file(
        self,
        parent_file_id: int,
        limit: int,
        search_data: str = "",
        search_mode: SearchMode = SearchMode.NORMAL,
        last_file_id: int = 0,
    ):
        data: dict = {"parentFileId": parent_file_id, "limit": limit}
        if search_data:
            data["searchData"] = search_data
        if search_mode:
            data["searchMode"] = search_mode
        if last_file_id:
            data["lastFileID"] = last_file_id
        return parse_response_data(
            requests.get(
                self.use_url("/api/v2/file/list"),
                data=data,
                headers=self.header,
            )
        )

    def mkdir(self, name: str, parent_id: int):
        return parse_response_data(
            requests.get(
                self.use_url("/upload/v1/file/mkdir"),
                data={"name": name, "parentID": parent_id},
                headers=self.header,
            )
        )

    def create(
        self,
        parent_file_id: int,
        filename: str,
        etag: str,
        size: int,
        duplicate: DuplicateMode = DuplicateMode.RENAME,
    ):
        data = {
            "parentFileID": parent_file_id,
            "filename": filename,
            "etag": etag,
            "size": size,
        }
        if duplicate:
            data["duplicate"] = duplicate.value
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/file/create"),
                data=data,
                headers=self.header,
            )
        )

    def get_upload_url(self, preupload_id: str, slice_no: int) -> str:
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/file/get_upload_url"),
                data={"preuploadID": preupload_id, "sliceNo": slice_no},
                headers=self.header,
            )
        )["presignedURL"]

    def list_upload_parts(self, preupload_id: str):
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/file/list_upload_parts"),
                data={"preuploadID": preupload_id},
                headers=self.header,
            )
        )

    def upload_complete(self, preupload_id: str):
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/file/upload_complete"),
                data={"preuploadID": preupload_id},
                headers=self.header,
            )
        )

    def upload_async_result(self, preupload_id: str):
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/file/upload_async_result"),
                data={"preuploadID": preupload_id},
                headers=self.header,
            )
        )

    def upload(self, parent_file_id: int, file_path: str):
        upload_data_parts = {}
        file_metadata = self.create(
            parent_file_id,
            os.path.basename(file_path),
            get_file_md5(file_path),
            os.stat(file_path).st_size,
        )
        if file_metadata["reuse"]:
            return
        num_slices = math.ceil(os.stat(file_path).st_size / file_metadata["sliceSize"])
        with open(file_path, "rb") as file_stream:
            for i in range(1, num_slices + 1):
                url = self.get_upload_url(file_metadata["preuploadID"], i)
                chunk = file_stream.read(file_metadata["sliceSize"])
                md5 = hashlib.md5(chunk).hexdigest()
                requests.put(url, data=chunk)
                upload_data_parts[i] = {
                    "md5": md5,
                    "size": len(chunk),
                }
        if not os.stat(file_path).st_size <= file_metadata["sliceSize"]:
            parts = self.list_upload_parts(file_metadata["preuploadID"])
            for i in parts["parts"]:
                part = i["partNumber"]
                if not (
                    upload_data_parts[int(part)]["md5"] == i["etag"]
                    and upload_data_parts[int(part)]["size"] == i["size"]
                ):
                    raise PacketLossError(i["partNumber"])
        self.upload_complete(file_metadata["preuploadID"])

    def rename(self, rename_dict: dict):
        rename_list = []
        for old_name in rename_dict.keys():
            new_name = rename_dict[old_name]
            rename_list.append(f"{old_name}|{new_name}")
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/file/rename"),
                data={"renameList": rename_list},
                headers=self.header,
            )
        )

    def move(self, file_id_list: list[int], to_parent_file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/file/move"),
                data={"fileIDs": file_id_list, "toParentFileID": to_parent_file_id},
                headers=self.header,
            )
        )

    def to_trashed(self, file_ids):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/file/trash"),
                data={"fileIDs": file_ids},
                headers=self.header,
            )
        )

    def recover(self, file_ids):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/file/recover"),
                data={"fileIDs": file_ids},
                headers=self.header,
            )
        )

    def delete(self, file_ids):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/file/delete"),
                data={"fileIDs": file_ids},
                headers=self.header,
            )
        )

    def detail(self, file_id):
        data = parse_response_data(
            requests.get(
                self.use_url("/api/v1/file/detail"),
                data={"fileID": file_id},
                headers=self.header,
            )
        )
        data["trashed"] = bool(data["trashed"])
        data["type"] = not data["type"]
        return data

    def download(self, file_id):
        return parse_response_data(
            requests.get(
                self.use_url("/api/v1/file/download"),
                data={"fileID": file_id},
                headers=self.header,
            )
        )["downloadURL"]



import json
import requests

from .utils.exceptions import AccessTokenError
from .utils.request import parse_response_data
from .abstracts import Requestable


class Share(Requestable):
    def create(
        self,
        share_name: str,
        share_expire: int,
        file_id_list: list,
        share_pwd: str = "",
        traffic_switch: bool = False,
        traffic_limit_switch: bool = False,
        traffic_limit: int = 0,
    ):
        data: dict = {
            "shareName": share_name,
            "shareExpire": share_expire,
            "fileIDList": file_id_list,
        }
        if share_pwd:
            data["sharePwd"] = share_pwd
        data = Share.apply_traffic_settings(
            data,
            traffic_switch,
            traffic_limit_switch,
            traffic_limit,
        )
        response = requests.post(
            self.use_url("/api/v1/share/create"),
            data=data,
            headers=self.header,
        )
        response_data = json.loads(response.text)
        parse_response_data(response, AccessTokenError)
        return {
            "shareID": response_data["data"]["shareID"],
            "shareLink": f"https://www.123pan.com/s/{response_data['data']['shareKey']}",
            "shareKey": response_data["data"]["shareKey"],
        }

    def list_info(
        self,
        share_id_list: list,
        traffic_switch: bool = False,
        traffic_limit_switch: bool = False,
        traffic_limit: int = 0,
    ):
        data: dict = Share.apply_traffic_settings(
            {"shareIdList": share_id_list},
            traffic_switch,
            traffic_limit_switch,
            traffic_limit,
        )
        return parse_response_data(
            requests.put(
                self.use_url("/api/v1/share/list/info"),
                data=data,
                headers=self.header,
            )
        )

    def list(self, limit: int, last_share_id: int = 0):
        data = {"limit": limit}
        if last_share_id:
            data["lastShareId"] = last_share_id
        return parse_response_data(
            requests.get(
                self.use_url("/api/v1/share/list"), data=data, headers=self.header
            )
        )

    @staticmethod
    def apply_traffic_settings(
        data: dict,
        traffic_switch: bool = False,
        traffic_limit_switch: bool = False,
        traffic_limit: int = 0,
    ) -> dict:
        data = data.copy()
        data["trafficSwitch"] = bool(traffic_switch) + 1  # True=1,False=0
        data["trafficLimitSwitch"] = bool(traffic_limit_switch) + 1
        if traffic_limit_switch and traffic_limit <= 0:
            raise ValueError("需要限制流量时，限制值必须大于0")
        return data



import hashlib, requests, os, math

from .utils.request import parse_response_data
from .utils.file_metadata import get_file_md5
from .utils.exceptions import PacketLossError
from .abstracts import Requestable
from .oss_source_copy import OSSSourceCopy
from .costants import DuplicateMode


class OSS(Requestable):
    def __init__(self, base_url, header):
        super().__init__(base_url, header)
        self.source_copy = OSSSourceCopy(base_url, header)

    def list_file(
        self,
        parent_file_id: int,
        limit: int = 0,
        start_time: int = 0,
        end_time: int = 0,
        last_file_id: int = 0,
    ) -> dict:
        data = {
            "parentFileId": parent_file_id,
            "type": 1,
        }
        if limit:
            data["limit"] = limit
        if start_time:
            data["startTime"] = start_time
        if end_time:
            data["endTime"] = end_time
        if last_file_id:
            data["lastFileId"] = last_file_id
        return parse_response_data(
            requests.get(
                self.use_url("/api/v1/oss/file/list"),
                data=data,
                headers=self.header,
            )
        )

    def mkdir(self, name: str, parent_id: int):
        return parse_response_data(
            requests.get(
                self.use_url("/upload/v1/oss/file/mkdir"),
                data={
                    "name": name,
                    "parentID": parent_id,
                    "type": 1,
                },
                headers=self.header,
            )
        )

    def create(
        self,
        preupload_id: int,
        filename: str,
        etag: str,
        size: int,
        duplicate: DuplicateMode = DuplicateMode.RENAME,
    ):
        data = {
            "parentFileID": preupload_id,
            "filename": filename,
            "etag": etag,
            "size": size,
            "type": 1,
        }
        if duplicate:
            data["duplicate"] = duplicate
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/oss/file/create"),
                data=data,
                headers=self.header,
            )
        )

    def get_upload_url(self, preupload_id: str, slice_index: int):
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/oss/file/get_upload_url"),
                data={"preuploadID": preupload_id, "sliceNo": slice_index},
                headers=self.header,
            )
        )["presignedURL"]

    def list_upload_parts(self, preupload_id: str) -> list[dict]:
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/oss/file/list_upload_parts"),
                data={"preuploadID": preupload_id},
                headers=self.header,
            )
        )

    def upload_complete(self, preupload_id: str):
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/oss/file/upload_complete"),
                data={"preuploadID": preupload_id},
                headers=self.header,
            )
        )

    def upload_async_result(self, preupload_id: str):
        return parse_response_data(
            requests.post(
                self.use_url("/upload/v1/oss/file/upload_async_result"),
                data={"preuploadID": preupload_id},
                headers=self.header,
            )
        )

    def upload(self, preupload_id: int, file_path: str):
        upload_data_parts = {}
        f = self.create(
            preupload_id,
            os.path.basename(file_path),
            get_file_md5(file_path),
            os.stat(file_path).st_size,
        )
        num_slices = math.ceil(os.stat(file_path).st_size / f["sliceSize"])
        with open(file_path, "rb") as fi:
            for part in range(1, num_slices + 1):
                url = self.get_upload_url(f["preuploadID"], part)
                chunk = fi.read(f["sliceSize"])
                md5 = hashlib.md5(chunk).hexdigest()
                requests.put(url, data=chunk)
                upload_data_parts[part] = {
                    "md5": md5,
                    "size": len(chunk),
                }
        if not os.stat(file_path).st_size <= f["sliceSize"]:
            parts = self.list_upload_parts(f["preuploadID"])
            for part in parts:
                if not (
                    upload_data_parts[part]["md5"] == part["etag"]
                    and upload_data_parts[part]["size"] == part["size"]
                ):
                    raise PacketLossError(part["partNumber"])
        self.upload_complete(f["preuploadID"])

    def move(self, file_id_list: list[int], to_parent_file_id: int):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/oss/file/move"),
                data={
                    "fileIDs": file_id_list,
                    "toParentFileID": to_parent_file_id,
                },
                headers=self.header,
            )
        )

    def delete(self, file_ids: list[int]):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/oss/file/delete"),
                data={"fileIDs": file_ids},
                headers=self.header,
            )
        )

    def detail(self, file_id: int):
        r = requests.post(
            self.use_url("/api/v1/oss/file/detail"),
            data={"fileID": file_id},
            headers=self.header,
        )
        data = parse_response_data(r)
        data["trashed"] = bool(data["trashed"])
        data["type"] = not data["type"]
        return data



import hashlib


def get_file_md5(file_path: str) -> str:
    md5_hash = hashlib.md5()
    with open(file_path, "rb") as file:
        for chunk in iter(lambda: file.read(4096), b""):
            md5_hash.update(chunk)
    return md5_hash.hexdigest()



import requests

from .utils.request import parse_response_data
from .abstracts import Requestable


class User(Requestable):
    def info(self) -> dict:
        return parse_response_data(
            requests.get(
                self.use_url("/api/v1/user/info"),
                headers=self.header,
            )
        )



import requests

from .utils.request import parse_response_data
from .abstracts import Requestable


class OfflineDownload(Requestable):
    def download(
        self,
        download_url: str,
        file_name: str = "",
        save_path: str = "",
        call_back_url: str = "",
    ):
        data = {"url": download_url}
        if file_name:
            data["fileName"] = file_name
        if save_path:
            data["savePath"] = save_path
        if call_back_url:
            data["callBackUrl"] = call_back_url
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/offline/download"),
                data=data,
                headers=self.header,
            )
        )

    def download_process(self, task_id):
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/offline/download/process"),
                data={"taskID": task_id},
                headers=self.header,
            )
        )



import requests

from .utils.exceptions import ClientKeyError
from .utils.request import parse_response_data
from .abstracts import Requestable


class Auth(Requestable):
    def get_access_token(
        self,
        client_id: str,
        client_secret: str,
    ) -> str:
        return parse_response_data(
            requests.post(
                self.use_url("/api/v1/access_token"),
                data={"clientID": client_id, "clientSecret": client_secret},
                headers=self.header,
            ),
            ClientKeyError,
        )["accessToken"]


class Requestable:
    base_url: str
    header: dict

    def __init__(self, base_url: str, header: dict):
        self.base_url = base_url if base_url.endswith("/") else f"{base_url}/"
        self.header = header

    def use_url(self, url: str) -> str:
        return f"{self.base_url}{url[1:] if url.startswith('/') else url}"
